@@ -4,7 +4,10 @@ This file contains most of the classes provided by chanim. Includes ChemObject,
 
 from typing import List, Union
 
+import numpy as np
+
 from .templates import ChemReactionTemplate, ChemTemplate
+from .utils import Arrows
 
 from manim.mobject.types.vectorized_mobject import VGroup, VMobject
 from manim.mobject.svg.tex_mobject import *
@@ -14,8 +17,10 @@ from manim.animation.creation import Write
 from manim.animation.composition import AnimationGroup
 from manim.mobject.geometry import DashedLine
 from manim.constants import *
-from manim.mobject.geometry import SmallDot
+from manim.mobject.geometry import Dot
 from manim.utils.color import YELLOW
+from manim.opengl import OpenGLMathTex
+from manim._config import logger
 
 
 def check_if_instance_change_if_not(obj, instance_of):
@@ -92,6 +97,75 @@ class ChemObject(MathTex):
         )
 
 
+class OpenGLChemObject(OpenGLMathTex):
+    """
+    `chanimlib.chem_objects.ChemObject`
+
+    Chemical Object
+    ===============
+
+    Uses `chemfig` to create two-dimensional molecular/atomic structures.
+    Only accepts `chemfig` commands.\n
+    Example:\n
+    >>> water = ChemObject("H_2O")    ## Linear Structure
+
+    or
+
+    >>> water = ChemObject("H[5]-O-H[-1]")      ## After repulsion
+
+    You can also set various prperties of the molecule using keyword arguments. See
+    `__init__` and page 7 of the manual (http://ctan.imsc.res.in/macros/generic/chemfig/chemfig-en.pdf)
+    """
+
+    # CONFIG = {"stroke_width": 2, "tex_template": ChemTemplate()}
+
+    def __init__(
+        self,
+        chem_code: str,
+        atom_sep: str = "2em",  ## all of these are the defaults in chemfig
+        chemfig_style: str = "",
+        atom_style: str = "",
+        angle_increment: int = 45,
+        bond_offset: str = "2pt",
+        double_bond_sep: str = "2pt",
+        node_style: str = "",
+        bond_style: str = "",
+        stroke_width: str = 5,
+        tex_template=ChemTemplate,
+        **kwargs,
+    ):
+        # digest_config(self, kwargs)
+        self.template: ChemTemplate = tex_template()
+        self.template.set_chemfig(
+            atom_sep=atom_sep,
+            chemfig_style=chemfig_style,
+            atom_style=atom_style,
+            angle_increment=angle_increment,
+            bond_offset=bond_offset,
+            double_bond_sep=double_bond_sep,
+            node_style=node_style,
+            bond_style=bond_style,
+        )
+        super().__init__(
+            "\\chemfig{%s}" % chem_code,
+            stroke_width=stroke_width,
+            tex_template=self.template,
+            **kwargs,
+        )
+
+    def set_ion_position(
+        self, string_number=0, e_index=0, final_atom_index=1, direction=LEFT
+    ):
+        """
+        This is to facilitate shifting of ionic and lewis electrons,
+        by `.move_to`'ing them to the direction you provide.
+        """
+
+        self[string_number][e_index].move_to(
+            self[final_atom_index].get_edge_center(direction) + direction * 0.2
+        )
+
+
 class ComplexChemIon(MathTex):
     """
     Mono-ionic Complexes
@@ -99,10 +173,23 @@ class ComplexChemIon(MathTex):
 
     # CONFIG = {"stroke_width": 2, "charge": ""}
 
-    def __init__(self, chem_code, stroke_width=2, charge="",tex_template=ChemTemplate(), **kwargs):
+    def __init__(
+        self,
+        chem_code,
+        stroke_width=2,
+        charge="",
+        tex_template=ChemTemplate(),
+        **kwargs,
+    ):
         # digest_config(self, kwargs)
         self.comp = "\chemleft[\chemfig{" + chem_code + "}\chemright]^{%s}" % charge
-        MathTex.__init__(self, self.comp, stroke_width=stroke_width,tex_template=tex_template, **kwargs)
+        MathTex.__init__(
+            self,
+            self.comp,
+            stroke_width=stroke_width,
+            tex_template=tex_template,
+            **kwargs,
+        )
 
 
 class ComplexChemCompound(MathTex):
@@ -177,22 +264,11 @@ class Reaction(Tex):
     #     "tex_template": ChemReactionTemplate(),
     # }
 
-    arrows = {
-        "forward": "->",
-        "backward": "<-",
-        "eq": "<=>",
-        "eq_fw": "<->>",
-        "eq_bw": "<<->",
-        "double": "<->",
-        "space": "0",
-        "split": "-U",
-    }
-
     def __init__(
         self,
-        reactants: List[str] = [],
-        products: List[str] = [],
-        arrow_type="forward",
+        reactants: List[str] = None,
+        products: List[str] = None,
+        arrow_type: Arrows = Arrows.forward,
         arrow_length=1,
         arrow_angle=0,
         arrow_style="",
@@ -237,16 +313,37 @@ class Reaction(Tex):
             debug=debug,
         )
 
-        self.reactants = reactants
-        self.products = products
-        # print(self.reactants,self.products)
-        self.arrow_type = arrow_type
+        if reactants is None:
+            logger.warning("No reactants provided, defaulting to empty list.")
+            self.reactants = []
+
+        if products is None:
+            logger.warning("No products provided, defaulting to empty list.")
+            self.products = []
+
+        if all([hasattr(reactants, "__iter__"), hasattr(reactants, "__iter__")]):
+            self.reactants = reactants
+            self.products = products
+        else:
+            raise TypeError("Reaction reactants and products must be None or iterable.")
+
+        ## Arrow stuff
+        if type(arrow_type) == Arrows:
+            self.arrow_type = arrow_type.value
+        elif arrow_type in Arrows.__members__.keys():  # handles old strings
+            self.arrow_type = Arrows[arrow_type].value
+        else:
+            logger.warning(
+                "Arrow type not recognised. Defaulting to Arrows.forward (->)"
+            )
+            self.arrow_type = Arrows.forward.value
+
         self.arrow_text_up = arrow_text_up
         self.arrow_text_down = arrow_text_down
         self.arrow_align_params = arrow_align_params
 
         self.equation = self.get_equation()
-        print(repr(self.equation))
+        # print(repr(self.equation))
 
         super().__init__(
             *self.equation,
@@ -264,13 +361,13 @@ class Reaction(Tex):
         if self.arrow_align_params != "":
             arrow = "\\arrow(%s){%s[%s][%s]}" % (
                 self.arrow_align_params,
-                self.arrows[self.arrow_type],
+                self.arrow_type,
                 self.arrow_text_up,
                 self.arrow_text_down,
             )
         else:
             arrow = "\\arrow{%s[%s][%s]}" % (
-                self.arrows[self.arrow_type],
+                self.arrow_type,
                 self.arrow_text_up,
                 self.arrow_text_down,
             )
@@ -280,8 +377,9 @@ class Reaction(Tex):
         # print(self.excluded_strings) ##for debugging
         r = [
             "\\chemfig{" + R + "}"
-            if R != self.reactants[-1] and len(self.reactants) != 1
-            else "\\chemfig{" + R + "}"
+            # if R != self.reactants[-1] and len(self.reactants) != 1
+            # else "\\chemfig{" + R + "}"
+            if "chemfig" not in R else R
             for R in self.reactants
         ]
 
@@ -289,8 +387,9 @@ class Reaction(Tex):
 
         p = [
             "\\chemfig{" + P + "}"
-            if P != self.products[-1] and len(self.products) != 1
-            else "\\chemfig{" + P + "}"
+            # if P != self.products[-1] and len(self.products) != 1
+            # else "\\chemfig{" + P + "}"
+            if "chemfig" not in P else P
             for P in self.products
         ]
 
@@ -321,7 +420,8 @@ class Reaction(Tex):
     def get_breakdown_dict(self):
         pass
 
-    def show(
+    # can't really think of a better name for this atm, and show will overwrite Mobject.show
+    def creation_anim(
         self,
         text_anim: Animation = Write,
         arrow_anim: Animation = FadeInFrom,
@@ -463,7 +563,7 @@ class ChemWithName(VMobject):
     inspired by `BraceLabel`
     """
 
-    CONFIG = {"label_constructor": TextMobject, "buff": 1}
+    CONFIG = {"label_constructor": Tex, "buff": 1}
 
     def __init__(
         self, chem, name, name_direction=DOWN, label_constructor=Tex, buff=1, **kwargs
@@ -471,7 +571,9 @@ class ChemWithName(VMobject):
 
         super().__init__(self, **kwargs)
 
-        if isinstance(chem, (ChemObject,ComplexChemIon,ComplexChemCompound,ChemAbove)):
+        if isinstance(
+            chem, (ChemObject, ComplexChemIon, ComplexChemCompound, ChemAbove)
+        ):
             self.chem = chem
         else:
             self.chem = ChemObject(chem, **kwargs)
@@ -586,7 +688,7 @@ class BondBreak(DashedLine):
 
 
 class ElectronPair(VGroup):
-    """Electron Pair: Two electrons (SmallDots) in a VGroup
+    """Electron Pair: Two electrons (Dots) in a VGroup
 
     Arguments:
         None -- Just use this as it is and change the kwargs if you want
@@ -595,7 +697,11 @@ class ElectronPair(VGroup):
     # CONFIG = dict(color=YELLOW, pair_buff=0.15)
 
     def __init__(self, color=YELLOW, pair_buff=0.15, **kwargs):
-        super().__init__(SmallDot(), SmallDot(), **kwargs)
+        super().__init__(
+            Dot(radius=DEFAULT_SMALL_DOT_RADIUS),
+            Dot(RADIUS=DEFAULT_SMALL_DOT_RADIUS),
+            **kwargs,
+        )
         self.arrange(RIGHT, buff=pair_buff).set_color(color)
 
 
